@@ -11,8 +11,13 @@
 #include <algorithm>
 #include <unordered_map>
 #include <string>
-#include <sstream>
+#if __cplusplus >= 201703L
+#include <string_view>
+#endif
 #include <initializer_list>
+#ifndef LO_USE_EXCEPTIONS
+#include <cassert>
+#endif
 
 /**
  * \file lo_cpp.h The liblo C++ wrapper
@@ -47,7 +52,7 @@
             std::unique_ptr<handler>(new handler_type<r ht>(h)));       \
         lo_method m = _add_method(path, types,                          \
             [](const char *path, const char *types,                     \
-               lo_arg **argv, int argc, void *msg,                      \
+               lo_arg **argv, int argc, lo_message msg,                 \
                void *data)->int                                         \
             {                                                           \
                 r1 (*static_cast<handler_type<r ht>*>(data)) args;      \
@@ -74,20 +79,30 @@ namespace lo {
     // "std::string", and "int".
     class string_type {
       public:
+        string_type(const string_type& s) { _s = s._s; }
         string_type(const char *s=0) { _s = s; }
         string_type(const std::string &s) { _s = s.c_str(); }
+#if __cplusplus >= 201703L
+        string_type(const std::string_view& s) {
+            if (s[s.length()]==0) _s = s.data();
+            else { _p.reset(new std::string(s)); _s = _p->c_str(); }
+        }
+#endif
         operator const char*() const { return _s; }
         std::string s() const { return _s?_s:""; }
         const char *_s;
+        std::unique_ptr<std::string> _p;
     };
 
     class num_string_type : public string_type {
       public:
-      num_string_type(const char *s) : string_type(s) {}
-      num_string_type(const std::string &s) : string_type(s) {}
-        num_string_type(int n) { std::ostringstream ss; ss << n;
-            _p.reset(new std::string(ss.str())); _s = _p->c_str(); }
-        std::unique_ptr<std::string> _p;
+        num_string_type(const char *s) : string_type(s) {}
+        num_string_type(const std::string &s) : string_type(s) {}
+#if __cplusplus >= 201703L
+        num_string_type(const std::string_view& s) : string_type(s) {}
+#endif
+        num_string_type(int n)
+          {_p.reset(new std::string(std::to_string(n))); _s = _p->c_str(); }
     };
 
 /*
@@ -125,7 +140,6 @@ namespace lo {
 #define LO_CHECK_BEFORE if (!is_valid()) throw Invalid();
 #define LO_CHECK_AFTER if (!is_valid()) throw Error();
 #else
-#include <cassert>
 #define LO_CHECK_BEFORE assert(is_valid());
 #define LO_CHECK_AFTER
 #endif
@@ -293,8 +307,8 @@ namespace lo {
         int set_tcp_nodelay(int enable)
           { LO_CHECK_BEFORE; return lo_address_set_tcp_nodelay(address, enable); }
 
-        int set_stream_slip(int enable)
-          { LO_CHECK_BEFORE; return lo_address_set_stream_slip(address, enable); }
+        int set_stream_slip(lo_slip_encoding encoding)
+          { LO_CHECK_BEFORE; return lo_address_set_stream_slip(address, encoding); }
 
         operator lo_address() const
             { return address; }
@@ -620,8 +634,9 @@ namespace lo {
         {
           LO_CHECK_BEFORE;
           for (auto &i : _handlers) {
-            std::remove_if(i.second.begin(), i.second.end(),
+            auto it = std::remove_if(i.second.begin(), i.second.end(),
                            [&](std::unique_ptr<handler>& h){return h->method == m;});
+            i.second.erase(it, i.second.end());
           }
           return lo_server_del_lo_method(server, m);
         }
@@ -743,7 +758,8 @@ namespace lo {
           : Server(0)
         { server_thread = lo_server_thread_new(port, err_h);
           if (server_thread)
-            server = lo_server_thread_get_server(server_thread); }
+            server = lo_server_thread_get_server(server_thread);
+          LO_CHECK_AFTER; }
 
         template <typename E>
         ServerThread(const num_string_type &port, E&& e)
